/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.vorgang.grpc.file.FileServiceGrpc.FileServiceImplBase;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsResponse;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsResponse;
import io.grpc.stub.StreamObserver;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
public class GrpcFileService extends FileServiceImplBase {

	@Autowired
	private FileService service;

	@Autowired
	private GrpcOzgFileMapper fileMapper;

	@Override
	public void getAttachments(GrpcGetAttachmentsRequest request, StreamObserver<GrpcGetAttachmentsResponse> responseObserver) {
		List<OzgFile> response = service.getAttachments(getVorgangId(request), getEingangId(request));

		responseObserver.onNext(GrpcGetAttachmentsResponse.newBuilder().addAllFile(fileMapper.map(response)).build());
		responseObserver.onCompleted();
	}

	private String getVorgangId(GrpcGetAttachmentsRequest request) {
		return Optional.of(request.getVorgangId()).map(StringUtils::trimToNull).orElseThrow(this::createMissingVorgangIdException);
	}

	private Optional<String> getEingangId(GrpcGetAttachmentsRequest request) {
		return Optional.of(request.getEingangId()).map(StringUtils::trimToNull);
	}

	@Override
	public void getRepresentations(GrpcGetRepresentationsRequest request, StreamObserver<GrpcGetRepresentationsResponse> responseObserver) {
		List<OzgFile> response = service.getRepresentations(getVorgangId(request), getEingangId(request));

		responseObserver.onNext(GrpcGetRepresentationsResponse.newBuilder().addAllFile(fileMapper.map(response)).build());
		responseObserver.onCompleted();
	}

	private String getVorgangId(GrpcGetRepresentationsRequest request) {
		return Optional.of(request.getVorgangId()).map(StringUtils::trimToNull).orElseThrow(this::createMissingVorgangIdException);
	}

	private Optional<String> getEingangId(GrpcGetRepresentationsRequest request) {
		return Optional.of(request.getEingangId()).map(StringUtils::trimToNull);
	}

	TechnicalException createMissingVorgangIdException() {
		return new TechnicalException("VorgangId is required");
	}
}
