/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Random;
import java.util.UUID;

import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.vorgang.files.FileId;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class IncomingFileTestFactory {

	public static final FileId ID = FileId.createNew();
	public static final String VENDOR_ID = UUID.randomUUID().toString();
	public static final String NAME = "TestDoc.docx";
	public static final String CONTENT_TYPE_STR = "application/xml";
	public static final byte[] CONTENT;
	public static final long SIZE;

	static {
		CONTENT = initContent();
		SIZE = CONTENT.length;
	}

	public static IncomingFile create() {
		return createBuilder().build();
	}

	public static IncomingFile.IncomingFileBuilder createBuilder() {
		return IncomingFile.builder()
				.id(ID)
				.vendorId(VENDOR_ID)
				.name(NAME)
				.contentType(CONTENT_TYPE_STR)
				.size(SIZE);
	}

	public static InputStream createContentStreamInMB(int sizeInMB) {
		int fileSize = sizeInMB * 1_000_000;

		return createContentStreamInByte(fileSize);
	}

	public static InputStream createContentStreamInByte(int fileSize) {
		return new ByteArrayInputStream(getFilledChunk(fileSize));
	}

	static byte[] getFilledChunk(int size) {
		Random r = new Random();
		byte[] chunk = new byte[size];
		for (int i = 0; i < chunk.length; i++) {
			chunk[i] = (byte) (r.nextInt('z' - 'a') + 'a');
		}
		return chunk;
	}

	private static byte[] initContent() {
		try (InputStream in = TestUtils.loadFile(NAME)) {
			byte[] fileData = new byte[in.available()];
			in.read(fileData);

			return fileData;
		} catch (IOException e) {
			throw new IllegalStateException("Error reading test file " + NAME);
		}
	}
}
